
WHITE = {255, 255, 255}
BLACK = {  0,   0,   0}
RED = {255, 63, 63}
BLUE = {63, 63, 255}
GREEN = {63, 255, 63}

function abs(x) return (x >= 0) and x or -x end
function bound(x, l, r) return math.max(l, math.min(x, r)) end

do
  Graphics = {
    lastColor = WHITE,
    scale = 1,
  }

  local fontset = [=[ !"#$%&'()*+,-./0123456789:;<=>?]=] ..
    [=[@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_]=] ..
    [=[`abcdefghijklmnopqrstuvwxyz{|}~]=]

  function Graphics:init()
    self.scale = math.floor(love.graphics.getWidth() / 160.0)
    self:loadFont("cgafont.png")
  end

  function Graphics:loadFont(name)
    self.fontimg = love.graphics.newImage(name)
    self.fontimg:setFilter("nearest", "nearest")
    self.font = love.graphics.newImageFont(self.fontimg, fontset)
    self.fontHeight = self.fontimg:getHeight()
    self.font:setLineHeight( self.fontHeight )
    love.graphics.setFont(self.font)
    return self
  end

  function Graphics:color(c, forced)
    if (c~=nil) and (forced or (c~=self.lastColor)) then
      self.lastColor = c
      love.graphics.setColor(c)
    end
    return self
  end

  function Graphics:rect(x, y, w, h)
    love.graphics.rectangle("fill", x, y, w, h)
    return self
  end

  function Graphics:thing(img, ox, oy, w, h)
    ox, oy = ox or 0, oy or 0
    for y = 1, #img do
      for x = 1, #img[y] do
        if img[y][x]==1 then
          self:rect(ox+(x-1)*w, oy+(y-1)*h, w, h)
        end
      end
    end
    return self
  end

  function Graphics:face(img, ox, oy)
    return self:thing(img, ox, oy, 16, 16)
  end

  function Graphics:icon(img, ox, oy)
    return self:thing(img, ox, oy, 8, 8)
  end

  function Graphics:nextScale()
    self.scale = (self.scale % 6) + 1
    love.graphics.setMode(self.scale*160, self.scale*160, false, true)
    return self
  end

  function Graphics:draw(fn, ...)
    self:color(WHITE, true)
    love.graphics.push()
    love.graphics.scale(self.scale, self.scale)
    fn(...)
    love.graphics.pop()
    return self
  end

  function Graphics:write(x, y, str, ...)
    str = str:format(...)
    if y == "center" then
      local lines = 0
      for _ in str:gmatch("[^\r\n]+") do lines = lines + 1 end
      y = floor((self.gameHeight-(self.fontHeight+2)*lines)/2)
    end
    for line in str:gmatch("[^\r\n]+") do
      local lx = (x == "center") and floor((self.gameWidth/2)-(line:len()*4)) or x
      love.graphics.print(line, lx, y)
      y = y + self.fontHeight + 2
    end
    return self
  end

  function Graphics:screenshot()
    local name = string.format("screenshot-%s.png", os.date("%Y%m%d-%H%M%S"))
    local shot = love.graphics.newScreenshot()
    shot:encode(name, "png")
    return self
  end
end

Faces = {
  happy = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 0, 1, 1, 1, 0, 1 },
    { 1, 1, 0, 0, 0, 1, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  thrilled = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 0, 0, 0, 1, 1 },
    { 1, 1, 0, 0, 0, 1, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  neutral = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 0, 0, 0, 0, 0, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  sad = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 0, 0, 0, 1, 1 },
    { 1, 0, 1, 1, 1, 1, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  left = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 0, 1, 1, 1, 1 },
    { 1, 1, 0, 1, 1, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 0, 0, 0, 0, 1, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  right = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 1, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 1, 0, 0, 0, 0 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
  dead = {
    { 0, 1, 1, 1, 1, 1, 0 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 0, 1, 0, 1, 1 },
    { 1, 1, 1, 1, 1, 1, 1 },
    { 1, 1, 0, 0, 0, 1, 1 },
    { 1, 0, 1, 1, 1, 0, 1 },
    { 0, 1, 1, 1, 1, 1, 0 },
  },
}

Icons = {
  heart = {
    { 0, 1, 0, 1, 0 },
    { 1, 1, 1, 1, 1 },
    { 1, 1, 1, 1, 1 },
    { 0, 1, 1, 1, 0 },
    { 0, 0, 1, 0, 0 },
  },
  drop = {
    { 0, 0, 1, 0, 0 },
    { 0, 0, 1, 0, 0 },
    { 0, 1, 1, 1, 0 },
    { 1, 1, 1, 0, 1 },
    { 0, 1, 1, 1, 0 },
  },
  note = {
    { 1, 1, 1, 0, 0 },
    { 0, 0, 1, 0, 0 },
    { 0, 0, 1, 1, 1 },
    { 0, 0, 1, 0, 1 },
    { 0, 0, 1, 1, 1 },
  },
}

do
  Game = {
    buttons = {
      { spacer = 17 },
      {
        text = "FEED",
        stats = {
          happy = 5,
          satiety = 10,
          ego = 1,
          love = 1,
        },
        icon = "note", iconColor = GREEN,
      },
      {
        text = "PRAISE",
        stats = {
          happy = 5,
          satiety = 0,
          ego = 10,
          love = 2,
        },
        icon = "drop", iconColor = BLUE,
      },
      {
        text = "HUG",
        stats = {
          happy = 5,
          satiety = 0,
          ego = 2,
          love = 5,
        },
        icon = "heart", iconColor = RED,
      },
      { spacer = 18 },
    },
  }

  function Game:init()
    self.state = {
      px = 16,
      py = 16,
      moveClock = 0,
      mode = "float",
      face = "happy",
      btnClick = false,
    }

    self.stats = {
      happy = 25,
      satiety = 10,
      ego = 10,
      love = 10,
    }
  end

  function Game:draw()
    Graphics:color(WHITE):face(Faces[self.state.face], self.state.px, self.state.py)
    local x = 2
    for i, btn in ipairs(self.buttons) do
      self:drawButton(x, btn)
      x = x + 2 + self:buttonWidth(btn)
    end
    if self.icon then
      Graphics:color(self.icon.color):icon(Icons[self.icon.image],
        self.icon.x, self.icon.y)
    end
  end

  function Game:buttonWidth(btn)
    if btn.spacer then return btn.spacer end
    return #btn.text*8+3
  end

  function Game:drawButton(x, btn)
    local fg, bg = BLACK, WHITE
    if self.state.btnClick == btn then
      fg, bg = WHITE, BLACK
    end
    Graphics:color(bg):rect(x, 147, self:buttonWidth(btn), 11)
    if btn.text then
      Graphics:color(fg):write(x+2, 149, btn.text)
    end
    return self
  end

  function Game:update(dt)
    if self.icon then
      self.icon.clock = self.icon.clock - dt
      if self.icon.clock <= 0 then
        self.icon = nil
      end
    end

    if self.state.mode == "dontwant" then
      self.state.moveClock = self.state.moveClock + dt
      if self.state.moveClock >= 4 then
        self.state.mode = "float"
        self.state.moveClock = 0
      elseif self.state.moveClock >= 3 then
        self.state.face = "right"
      elseif self.state.moveClock >= 2 then
        self.state.face = "left"
      elseif self.state.moveClock >= 1 then
        self.state.face = "right"
      else
        self.state.face = "left"
      end
    end
    if self.state.mode == "gotstuff" then
      self.state.moveClock = self.state.moveClock + dt
      if self.state.moveClock >= 1.5 then
        self.state.mode = "float"
        self.state.moveClock = 0
      end
    end
    if self.state.mode == "float" then
      for k, v in pairs(self.stats) do
        self.stats[k] = math.max(0, v - dt)
      end
      if self.stats.happy >= 20 then
        self.state.face = "happy"
      elseif self.stats.happy >= 10 then
        self.state.face = "neutral"
      elseif self.stats.happy > 0 then
        self.state.face = "sad"
      else
        self.state.face = "dead"
      end
      if self.stats.happy > 0 then
        self.state.moveClock = self.state.moveClock + dt
        if self.state.moveClock > 0.5 then
          self.state.moveClock = self.state.moveClock - 0.5
          angle = math.random() * 2 * math.pi
          local step = self.stats.happy * 0.5
          self.state.px = bound( self.state.px + step*math.cos(angle), 0, 48)
          self.state.py = bound( self.state.py + step*math.sin(angle), 0, 32)
        end
      end
    end
  end

  function Game:click(mx, my)
    if (my >= 147) and (my <= 157) then
      local bx = 2
      for i, btn in ipairs(self.buttons) do
        local w = self:buttonWidth(btn)
        if btn.text and (mx >= bx) and (mx < (bx+w)) then
          self.state.btnClick = btn
        end
        bx = bx + 2 + w
      end
    end
  end

  function Game:unclick(mx, my)
    if (my >= 147) and (my <= 157) then
      local bx = 2
      for i, btn in ipairs(self.buttons) do
        local w = self:buttonWidth(btn)
        if btn.text and (mx >= bx) and (mx < (bx+w)) then
          if self.state.btnClick == btn then
            self:apply(btn)
          end
        end
        bx = bx + 2 + w
      end
    end
    self.state.btnClick = nil
  end

  function Game:apply(btn)
    if self.state.mode ~= "float" then return end

    local newstats = {}
    local toobig = false
    for k, v in pairs(self.stats) do
      newstats[k] = v + btn.stats[k]
      toobig = toobig or (newstats[k] > 40)
    end
    if toobig then
      self.state.mode = "dontwant"
      self.state.moveClock = 0
    else
      for k, _ in pairs(self.stats) do
        self.stats[k] = newstats[k]
      end
      self.state.mode = "gotstuff"
      self.state.moveClock = 0
      self.state.face = "thrilled"
      self.icon = {
        image = btn.icon,
        color = btn.iconColor,
        clock = 1.5,
        x=0, y=0,
      }
      if self.state.px <= 24 then
        self.icon.x = 120
      end
    end
  end
end

love.load = function(args)

  Graphics:init()
  Game:init()

  function love.keypressed(k, u)
    if k == "f2" then
      Graphics:screenshot()
    elseif k == "f9" then
      return love.load({})
    elseif k == "f5" then
      Graphics:nextScale()
    elseif k == "f10" then
      love.event.quit()
    end
  end

  function love.draw()
    Graphics:draw(Game.draw, Game)
  end

  function love.update(dt)
    Game:update(dt)
  end

  function love.mousepressed(x, y, button)
    if button == "l" then
      Game:click(math.floor(x/Graphics.scale), math.floor(y/Graphics.scale))
    end
  end

  function love.mousereleased(x, y, button)
    if button == "l" then
      Game:unclick(math.floor(x/Graphics.scale), math.floor(y/Graphics.scale))
    end
  end

  collectgarbage()
end

